local rttimanager = require "venuscore.rtti.rttimanager"
local venuscore = require "libvenuscore"



local Object = {}
Object.__index = Object
Object.__members = {}


function Object:new()
end


function Object:extend(name)
  local cls = {}
  for k, v in pairs(self) do
    if k:find("__") == 1 then
      cls[k] = v
    end
  end
  cls.__index = cls
  cls.super = self
  cls.__members = {}--初始化导出成员变量表
  for _, v in ipairs(self.__members) do 
      table.insert(cls.__members, v)
  end
  setmetatable(cls, self)
  if name then
    cls.__name = name;
    cls.__typeid = venuscore.Utility:HashCode(name)
    rttimanager:RegisterClass(name, cls);
  end  
  return cls
end


function Object:implement(...)
  for _, cls in pairs({...}) do
    for k, v in pairs(cls) do
      if self[k] == nil and type(v) == "function" then
        self[k] = v
      end
    end
  end
end


function Object:is(T)
  local mt = getmetatable(self)
  while mt do
    if mt == T then
      return true
    end
    mt = getmetatable(mt)
  end
  return false
end


function Object:__tostring()
  return "Object"
end


function Object:__call(...)
  local obj = setmetatable({}, self)
  obj:new(...)
  return obj
end

function Object:cast(obj)
  setmetatable(obj, self)
  obj.__index = Object
  return obj
end

function Object:clone()
  local obj = table.deepcopy(self);
  setmetatable(obj, self)
  obj.__index = Object
  return obj
end

--对齐c++接口
function Object:GetTypeName()
  return self.__name;
end

function Object:GetTypeID()
  return self.__typeid;
end

function Object:MemberRegister(name)
  --注册的成员变量名称不能重复
  --使用数组，遍历的时候更高效，所以需要注册的时候检查重复
  for _, value in ipairs(self.__members) do
    assert(value ~= name);
  end
  table.insert(self.__members, name);
end

return Object
